From 8f3ad4bd52390f6ef747004f3967100e2f59cf2b Mon Sep 17 00:00:00 2001
From: Shugo Maeda <shugo@ruby-lang.org>
Date: Wed, 1 Sep 2010 12:00:21 +0900
Subject: [PATCH 23/51] avoid SEGV on Mac.  https://gist.github.com/f8d8601218782cd04009

---
 vm.c            |    4 ++--
 vm_eval.c       |    2 +-
 vm_insnhelper.c |    8 +++++++-
 3 files changed, 10 insertions(+), 4 deletions(-)

diff --git a/vm.c b/vm.c
index 0634cf0..e171a5c 100644
--- a/vm.c
+++ b/vm.c
@@ -70,7 +70,7 @@ static inline VALUE
 rb_vm_set_finish_env(rb_thread_t * th)
 {
     vm_push_frame(th, 0, VM_FRAME_MAGIC_FINISH,
-		  Qnil, rb_cObject, th->cfp->lfp[0], 0,
+		  Qnil, Qnil, th->cfp->lfp[0], 0,
 		  th->cfp->sp, 0, 1);
     th->cfp->pc = (VALUE *)&finish_insn_seq[0];
     return Qtrue;
@@ -1784,7 +1784,7 @@ th_init2(rb_thread_t *th, VALUE self)
 
     th->cfp = (void *)(th->stack + th->stack_size);
 
-    vm_push_frame(th, 0, VM_FRAME_MAGIC_TOP, Qnil, rb_cObject, 0, 0,
+    vm_push_frame(th, 0, VM_FRAME_MAGIC_TOP, Qnil, Qnil, 0, 0,
 		  th->stack, 0, 1);
 
     th->status = THREAD_RUNNABLE;
diff --git a/vm_eval.c b/vm_eval.c
index 353414a..2bca7db 100644
--- a/vm_eval.c
+++ b/vm_eval.c
@@ -163,7 +163,7 @@ vm_call_super(rb_thread_t *th, int argc, const VALUE *argv)
     rb_method_entry_t *me;
     rb_control_frame_t *cfp = th->cfp;
 
-    if (!cfp->iseq) {
+    if (!cfp->iseq && !NIL_P(cfp->klass)) {
 	klass = RCLASS_SUPER(cfp->klass);
 
 	if (klass == 0) {
diff --git a/vm_insnhelper.c b/vm_insnhelper.c
index edeefb8..bd1491e 100644
--- a/vm_insnhelper.c
+++ b/vm_insnhelper.c
@@ -66,7 +66,13 @@ vm_push_frame(rb_thread_t * th, const rb_iseq_t * iseq,
 	cfp->klass = klass;
     }
     else {
-	cfp->klass = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp)->klass;
+	rb_control_frame_t *prev_cfp = RUBY_VM_PREVIOUS_CONTROL_FRAME(cfp);
+	if (RUBY_VM_CONTROL_FRAME_STACK_OVERFLOW_P(th, prev_cfp)) {
+	    cfp->klass = Qnil;
+	}
+	else {
+	    cfp->klass = prev_cfp->klass;
+	}
     }
 
 #define COLLECT_PROFILE 0
-- 
1.7.0.4

